#!/bin/sh
#for full effect this must be sourced (. enter_test_mode.sh), not just run
#pass -n to just set the name/email variables, nothing to also generate test keys
#uses rsa1024 for quicker generation, since test keys aren't meant to be secure (use generate_key.sh for real keys)
if whoami | grep root ; then
    #for chroot testing
    adduser user --disabled-password --gecos aaa
    echo "Switch to user 'user' and run this again"
    exit 0
fi
export DEBEMAIL=test1@rnpalmer-laptop
export DEBFULLNAME="Test key 1 - do not trust"
#done this way because 'exit 0' exits the entire login shell (and hence re-runs startup.sh, changing the envvars back) while Ctrl+C aborts the current command but keeps going with the rest
if [ "$1" = "-n" ] ; then true; else
    #unmount media, hiding errors if they aren't present
    sudo umount /mnt/keys1 2>/dev/null
    sudo umount /mnt/keys2 2>/dev/null
    sudo umount /mnt/keys3 2>/dev/null
    if findmnt /lib/live/mount/medium 2>&1 | grep -e "/dev/sr0" >/dev/null ; then true; else
        sudo umount /media/cdrom 2>/dev/null
    fi
    echo "Remove real key media, insert test key media (if required) and press Enter"
    python3 -c "input()"
    read_keys.sh
    mkdir -p ~/othergpgtest
    #generate a to-be-signed key
    #the passphrase is there to make sure nothing accidentally assumes we have a private key we wouldn't have for a real signature
    GNUPGHOME=~/othergpgtest gpg --pinentry-mode loopback --passphrase 'test' --quick-generate-key 'Test key 2 - do not trust <test2@rnpalmer-laptop>' rsa1024
    KEY_FINGERPRINT=$(GNUPGHOME=~/othergpgtest gpg --list-keys test2@rnpalmer-laptop | grep -oE -e "[0-9A-F]{40,}")
    GNUPGHOME=~/othergpgtest gpg --pinentry-mode loopback --passphrase 'test' --quick-add-key $KEY_FINGERPRINT rsa1024 encr 2y
    GNUPGHOME=~/othergpgtest gpg --export --output ~/othergpgtest/key_to_sign.gpg
    #generate 'our' key if it doesn't already exist
    if gpg --list-keys | grep -e test1 ; then true; else
        if gpg --quick-generate-key "$DEBFULLNAME <$DEBEMAIL>" rsa1024 default 2y ; then
            KEY_FINGERPRINT=$(gpg --list-keys test1@rnpalmer-laptop | grep -oE -e "[0-9A-F]{40,}")
            gpg --quick-add-key $KEY_FINGERPRINT rsa1024 sign 2y
            gpg --quick-add-key $KEY_FINGERPRINT rsa1024 encr 2y
            gpg --quick-add-key $KEY_FINGERPRINT rsa1024 auth 2y
        else
            #if we get to here, we are probably in a chroot (no pinentry) - use an unprotected test key
            gpg --pinentry-mode loopback --passphrase '' --quick-generate-key "$DEBFULLNAME <$DEBEMAIL>" rsa1024 default 2y
            KEY_FINGERPRINT=$(gpg --list-keys test1@rnpalmer-laptop | grep -oE -e "[0-9A-F]{40,}")
            gpg --pinentry-mode loopback --passphrase '' --quick-add-key $KEY_FINGERPRINT rsa1024 sign 2y
            gpg --pinentry-mode loopback --passphrase '' --quick-add-key $KEY_FINGERPRINT rsa1024 encr 2y
            gpg --pinentry-mode loopback --passphrase '' --quick-add-key $KEY_FINGERPRINT rsa1024 auth 2y
        fi
    fi
    echo "For key signing tests, the to-be-signed key is in ~/othergpgtest(/key_to_sign.gpg) and has passphrase 'test'"
fi
