#!/bin/sh
#This will ask for the passphrase at every subkey
#We can't use the obvious equivalent for extending expiry, as gpg --quick-set-expire will only act on the primary (it rejects non-primary fingerprints)
set -e
if [ \( -e /mnt/keys1 -o -e /mnt/keys2 -o -e /mnt/keys3 \) -a -e /mnt/transfer ] ; then true; else
    echo "Private and/or public USB media missing - continue (y/n)?"
    python3 -c "import sys;sys.exit(1 if input('')!='y' else 0)" || exit 1
fi
for KEY_LOCATION in /mnt/keys1 /mnt/keys2 /mnt/keys3 /media/cdrom ; do
    if [ -e $KEY_LOCATION/keys ] ; then
        echo "/keys exists, aborting to avoid overwrite (delete it or disable this check if intentionally reusing media)"
        exit 1
    fi
done
gpg --quick-generate-key "$DEBFULLNAME <$DEBEMAIL>" rsa4096 default 2y
KEY_FINGERPRINT=$(gpg --list-keys $DEBEMAIL | grep -oE -e "[0-9A-F]{40,}")
gpg --quick-add-key $KEY_FINGERPRINT rsa4096 sign 2y
gpg --quick-add-key $KEY_FINGERPRINT rsa4096 encr 2y
gpg --quick-add-key $KEY_FINGERPRINT rsa4096 auth 2y
if [ -e /mnt/transfer ] ; then
    gpg --output /mnt/transfer/my_public_key.gpg --export
    cp -r ~/.gnupg/openpgp-revocs.d -T /mnt/transfer/emergency_revoke
    echo "Written public key and revocation certificate to /mnt/transfer"
fi
#update trustdb
gpg --list-keys
#these special files break copying
rm -f ~/.gnupg/S.gpg-agent*
for KEY_LOCATION in /mnt/keys1 /mnt/keys2 /mnt/keys3 ; do
    if [ -e $KEY_LOCATION ] ; then
        mkdir $KEY_LOCATION/keys
        cp -r ~/.gnupg -T $KEY_LOCATION/keys/1
        cp -r ~/.gnupg -T $KEY_LOCATION/keys/2
        cp -r ~/.gnupg -T $KEY_LOCATION/keys/bak
        echo "Written keys to $KEY_LOCATION/keys"
    fi
done
if findmnt /lib/live/mount/medium 2>&1 | grep -e "/dev/sr0" ; then true; else
    sudo umount /media/cdrom || true
    echo "If you accidentally miss the next (writing to CD) prompt in random typing, comment out the 4 key generation lines and run this again"
    while true; do
        echo "Write to a(nother) CD/DVD (y/n)? Insert it before replying y; existing keys on it may be destroyed without warning"
        python3 -c "import sys;sys.exit(1 if input('')!='y' else 0)" || exit 0
        #This leaves alone files outside the /keys/* directories, but deletes the boot sector, so a combined boot+keys disc doesn't work; I haven't tried to fix this as on my hardware (starting with a DVD in doesn't _reliably_ boot from it) it isn't a very secure idea anyway
        xorriso -abort_on FATAL -assert_volid KEYS FATAL -dev /dev/sr0 -volid KEYS -update_r /home/user/.gnupg /keys/1 -update_r /home/user/.gnupg /keys/2 -update_r /home/user/.gnupg /keys/bak
        #verify
        read_keys.sh
        sudo umount /media/cdrom || true
        sudo eject /dev/sr0 || true
    done
fi

