#!/usr/bin/python3
#To check (in the main (online) system, not the chroot/CD) if live-build fails with a missing package:
# - Has something changed package name (e.g. the ABI number in linux-image-*) or dependencies? If so, update *_stage_packages
# - Run apt-get update
# - Rerun this

import os
import os.path
import hashlib
import re
import glob
import subprocess
import logging
import datetime
import itertools
import shutil
import gzip


mirror_root='/mnt/transfer/aptmirror'
if not os.path.exists(mirror_root):
    mirror_root='/media/rnpalmer/transfer/aptmirror'
apt_root='/'
checker_root='/'
archive_keyring=checker_root+'/usr/share/keyrings/debian-archive-keyring.gpg'
dist_name='stretch'#as this uses apt-get downloads, it can only fetch the same release as the running system, and should be preceded by 'apt-get update'
arch_name='amd64'
#we can't use just the apt lists, as apt checks a plain Packages against the .gz hash (bug?) and hence rejects it
main_url='http://ftp.uk.debian.org/debian/'
security_url='http://security.debian.org/debian-security/'
do_download=os.path.exists(checker_root+"/usr/bin/wget")
#these are the packages it will download if necessary - it will also include any others available locally
bootstrap_stage_packages="""adduser apt base-files base-passwd bash bsdutils coreutils dash debconf debian-archive-keyring debianutils diffutils dpkg e2fslibs e2fsprogs findutils gcc-6-base gpgv grep gzip hostname init-system-helpers libacl1 libapt-pkg5.0 libattr1 libaudit1 libaudit-common libblkid1 libbz2-1.0 libc6 libcap-ng0 libc-bin libcomerr2 libdb5.3 libdebconfclient0 libfdisk1 libgcc1 libgcrypt20 libgpg-error0 liblz4-1 liblzma5 libmount1 libncursesw5 libpam0g libpam-modules libpam-modules-bin libpam-runtime libpcre3 libselinux1 libsemanage1 libsemanage-common libsepol1 libsmartcols1 libss2 libstdc++6 libsystemd0 libtinfo5 libudev1 libustr-1.0-1 libuuid1 login lsb-base mawk mount multiarch-support ncurses-base ncurses-bin passwd perl-base sed sensible-utils sysvinit-utils tar tzdata util-linux zlib1g""".split(' ')#these must be available in the plain-stretch version, as well as any -security
all_stage_packages=bootstrap_stage_packages+"""dctrl-tools alsa-utils binutils bsdmainutils busybox bzip2 console-setup console-setup-linux cpio cryptsetup cryptsetup-bin dbus debootstrap devscripts dh-python dmsetup dpkg-dev eject firmware-linux-free git git-man gnupg gnupg-agent groff-base init initramfs-tools initramfs-tools-core kbd keyboard-configuration klibc-utils kmod less libapparmor1 libasound2 libasound2-data libassuan0 libbsd0 libburn4 libc-l10n libcap2 libclass-method-modifiers-perl libclass-methodmaker-perl libconvert-binhex-perl libcryptsetup4 libcurl3-gnutls libdata-perl-perl libdbus-1-3 libdbus-glib-1-2 libdevel-globaldestruction-perl libdevmapper1.02.1 libdpkg-perl liberror-perl libexpat1 libexporter-tiny-perl libffi6 libfftw3-single3 libfile-homedir-perl libfile-which-perl libgdbm3 libglib2.0-0 libgmp10 libgnupg-interface-perl libgnutls30 libgomp1 libgssapi-krb5-2 libhogweed4 libhx28 libicu57 libidn11 libidn2-0 libimport-into-perl libio-socket-ssl-perl libip4tc0 libisoburn1 libisofs6 libjte1 libk5crypto3 libkeyutils1 libklibc libkmod2 libkrb5-3 libkrb5support0 libksba8 libldap-2.4-2 libldap-common liblist-moreutils-perl liblocale-gettext-perl libmailtools-perl libmd0 libmime-tools-perl libmodule-runtime-perl libmoo-perl libmoox-handlesvia-perl libmoox-late-perl libmpdec2 libncurses5 libnet-idn-encode-perl libnet-smtp-ssl-perl libnet-ssleay-perl libnettle6 libnewt0.52 libnghttp2-14 libnpth0 libp11-kit0 libpam-mount libparams-classify-perl libperl5.24 libpipeline1 libpopt0 libprocps6 libprotobuf10 libpsl5 libpython-stdlib libpython2.7-minimal libpython2.7-stdlib libpython3-stdlib libpython3.5-minimal libpython3.5-stdlib libqb0 libreadline7 librole-tiny-perl librtmp1 libsamplerate0 libsasl2-2 libsasl2-modules-db libseccomp2 libslang2 libsodium18 libsqlite3-0 libssh2-1 libssl1.1 libstrictures-perl libsub-exporter-progressive-perl libtasn1-6 libterm-readkey-perl libtext-template-perl libtimedate-perl libtype-tiny-perl libunistring0 libusb-1.0-0 libusbguard0 libxml2 linux-base linux-image-4.9.0-6-amd64 linux-image-amd64 live-boot live-boot-initramfs-tools live-build live-config live-config-systemd live-tools locales make man-db mime-support nano netbase patch perl perl-modules-5.24 perl-openssl-defaults pinentry-curses procps python python-minimal python2.7 python2.7-minimal python3 python3-minimal python3.5 python3.5-minimal qprint readline-common rsync scdaemon signing-party sudo systemd systemd-sysv udev usbguard user-setup uuid-runtime wamerican wget whiptail xkb-data xorriso xz-utils liblzo2-2 squashfs-tools memtest86+ isolinux syslinux-common dosfstools efibootmgr gettext-base grub-common grub-efi-amd64-bin grub-efi-ia32-bin libefiboot1 libefivar1 libfreetype6 libfuse2 libpng16-16 mtools zsync""".split(' ')#not all of these end up in the final image, but they are used at some point

def file_sha256(fname):
    with open(fname,'rb') as fd:
        hash1=hashlib.sha256()
        block='nonempty'
        while len(block)>0:
            block=fd.read(10**7)#don't run out of memory on very large files
            hash1.update(block)
        return hash1.hexdigest()
#move existing contents to /tmp to mark it as "unchecked"
shutil.rmtree(mirror_root+'/tmp',ignore_errors=True)
dirs_to_move=glob.glob(mirror_root+'/*')
os.makedirs(mirror_root+'/tmp')
for d in dirs_to_move:
    os.rename(d,mirror_root+'/tmp/'+os.path.basename(d.rstrip('/')))
if glob.glob(mirror_root+'/*')!=[mirror_root+'/tmp']:
    raise OSError("Failed to move old contents")
#fetch indices
for url1 in [main_url+'dists/'+dist_name+'/',main_url+'dists/'+dist_name+'-updates/',security_url+'dists/'+dist_name+'/updates/']:
    for urlfull in [url1+'Release',url1+'InRelease',url1+'Release.gpg',url1+'main/binary-'+arch_name+'/Packages.gz',url1+'main/i18n/Translation-en.bz2',url1+'main/binary-'+arch_name+'/Packages.xz']:#,url1+'/main/source/Sources.gz',url1+'/main/source/Sources.xz']:
        temp_filename=mirror_root+'/tmp'+urlfull.split('debian.org',1)[1]
        os.makedirs(os.path.dirname(temp_filename),exist_ok=True)
        if do_download:
            if subprocess.run([checker_root+"/usr/bin/wget",'-x','-nH','-N',urlfull],cwd=mirror_root+'/tmp').returncode!=0:
                if os.path.exists(temp_filename) and os.stat(temp_filename).st_size==0:#trying to download a nonexistent file gives 0-length file
                    os.unlink(temp_filename)
#verify indices and files
release_files=[f for f in glob.glob(mirror_root+'/tmp/**/*Release',recursive=True) if 'main' not in f]
packageindex_sha256={}
for f in release_files:
    fs=f+'.gpg'
    fbase=os.path.dirname(f)
    if subprocess.run([checker_root+"/usr/bin/gpgv","--keyring",archive_keyring,"--weak-digest","SHA1"]+([fs] if os.path.exists(fs) else [])+[f]).returncode==0:
        with open(f,'r') as fd:
            is_sha256=False
            for line in fd:
                if is_sha256:
                    r=re.fullmatch(" ([0-9a-f]{64})\s+[0-9]+\s(.*)\n",line)
                    if r:
                        pfhash,pfname=r.groups()
                        packageindex_sha256[fbase+'/'+pfname]=pfhash
                    else:
                        is_sha256=False
                if line=='SHA256:\n':
                    is_sha256=True
                if line.startswith('Valid-Until:'):
                    expiry=datetime.datetime.strptime(line,'Valid-Until: %a, %d %b %Y %H:%M:%S %Z\n')
                    if expiry<datetime.datetime.now():
                        raise RuntimeError("Release file outdated: "+f+" "+line+" Warning: running mirror_check_update again will delete *everything* on this local mirror, then try to do a possibly large download to re-create it")
                        #to avoid this, move the contents of /tmp back up to /, then re-run this with network access
                        #note that disabling this check just makes apt error out instead
        target_filename=os.path.join(mirror_root,os.path.relpath(f,start=mirror_root+'/tmp'))
        os.renames(f,target_filename)
        if os.path.exists(fs):
            os.renames(fs,target_filename+'.gpg')
    else:
        logging.error("Release file bad signature: "+f)
        os.unlink(f)
index_files={f for f in glob.glob(mirror_root+'/tmp/**/*',recursive=True) if os.path.isfile(f) and f not in release_files}
while index_files:
    f=index_files.pop()
    fhash=file_sha256(f)
    if f in packageindex_sha256:
        if packageindex_sha256[f]==fhash:
            target_filename=os.path.join(mirror_root,os.path.relpath(f,start=mirror_root+'/tmp'))
            if f.endswith('.bz2'):
                subprocess.run([checker_root+"/bin/bunzip2","-f","-k",f])
                index_files.add(f[:-4])
            if f.endswith('.gz'):
                subprocess.run([checker_root+"/bin/gunzip","-f","-k",f])
                index_files.add(f[:-3])
            os.renames(f,target_filename)
        else:
            logging.error("Index file bad hash: {} expected {} found {}".format(f,packageindex_sha256[f],fhash))
            os.unlink(f)
#get and verify .debs
packageindex_files=glob.glob(mirror_root+'/**/Packages.gz',recursive=True)
for f in packageindex_files:
    fhash=file_sha256(f)
    if os.path.join(mirror_root+'/tmp',os.path.relpath(f,start=mirror_root)) in packageindex_sha256:
        if packageindex_sha256[os.path.join(mirror_root+'/tmp',os.path.relpath(f,start=mirror_root))]==fhash:
            dist_path1=os.path.relpath(f,start=mirror_root).split('/')[0]
            with gzip.open(f,'rt',encoding='UTF-8') as fd:
                current_package={}
                for line in itertools.chain(fd,['\n']):#ensure the last package gets processed
                    for fieldname in ["Package","Version","Filename","SHA256","Architecture"]:
                        r=re.fullmatch(fieldname+': (.*)\n',line)
                        if r:
                            current_package[fieldname]=r.group(1)
                    if line=='\n' and current_package:#empty line=end of package
                        deb_tmpname=mirror_root+'/tmp/'+dist_path1+'/'+current_package['Filename']
                        if not os.path.exists(deb_tmpname):
                            #include _all_ packages from the apt cache
                            deb_aptcache=(apt_root+'/var/cache/apt/archives/'+current_package['Package']+'_'+current_package['Version'].replace(':','%3a')+'_'+current_package['Architecture']+'.deb').replace('//','/')
                            if os.path.exists(deb_aptcache):
                                os.makedirs(os.path.dirname(deb_tmpname),exist_ok=True)
                                shutil.copyfile(deb_aptcache,deb_tmpname)
                            #if that doesn't have a package we need, download it
                            elif current_package['Package'] in all_stage_packages and do_download and not os.path.exists(deb_tmpname):
                                os.makedirs(os.path.dirname(deb_tmpname),exist_ok=True)
                                if subprocess.run([checker_root+'/usr/bin/apt-get','download',current_package['Package']],cwd=os.path.dirname(deb_tmpname)).returncode!=0:#using apt-get and not wget to get the updated version if any
                                    logging.warning("Fetching failed: "+str(current_package))
                                if current_package['Package'] in bootstrap_stage_packages and re.search("/"+dist_name+"/[^u]",f) and not os.path.exists(deb_tmpname):
                                    logging.warning("Fetching possibly outdated package for bootstrap: "+str(current_package))
                                    if subprocess.run([checker_root+'/usr/bin/apt-get','download',current_package['Package']+'='+current_package['Version']],cwd=os.path.dirname(deb_tmpname)).returncode!=0:
                                        logging.warning("Fetching failed: "+str(current_package))
                                if ':' in current_package['Version']:#remove epoch from file name
                                    deb_tmpname0=os.path.dirname(deb_tmpname)+'/'+current_package['Package']+'_'+current_package['Version'].replace(':','%3a')+'_'+current_package['Architecture']+'.deb'
                                    if os.path.exists(deb_tmpname0):
                                        os.rename(deb_tmpname0,deb_tmpname)
                        if os.path.exists(deb_tmpname):
                            debhash=file_sha256(deb_tmpname)
                            if debhash==current_package['SHA256']:
                                target_filename=mirror_root+'/'+dist_path1+'/'+current_package['Filename']
                                os.renames(deb_tmpname,target_filename)
                            else:
                                logging.error(".deb file bad hash: {} expected {} found {}".format(deb_path,current_package['SHA256'],debhash))
                                os.unlink(deb_tmpname)
                        current_package={}
        else:
            logging.error("Packages file bad hash: {} expected {} found {}".format(f,packageindex_sha256[f],fhash))
            os.unlink(f)
    else:
        logging.warning("Packages file not listed in Release files - ignoring: "+f)
        os.unlink(f)
remaining_files=[f for f in glob.glob(mirror_root+'/tmp/**/*') if os.path.isfile(f)]
if remaining_files:
    logging.warning("Leftover files (maybe outdated packages?): "+str(remaining_files))
shutil.rmtree(mirror_root+'/tmp',ignore_errors=True)
