#!/bin/sh
#Build process (requires patched /usr/live/build/chroot_devpts to accept local mirror, see ../share/chroot_local_mirror.patch):
# 1) Run mirror_check_update.py online as normal user
# 2) Run this offline as root (from either the main system or a previous copy of the liveCD)
#It is normal to get 'wget: unable to resolve' and similar 'no network' errors during the second step - ignore them
set -e
#is this a from-main-system build or a from-inside-image build?
if [ -e /usr/local/share/image1 ]; then
    SOURCE_PATH=/usr/local/share/image1
    FROMSELF_BUILD=true
    MIRROR_ROOT=/mnt/transfer/aptmirror
    USER1=user
    if findmnt /lib/live/mount/medium 2>&1 | grep -e "/dev/sr0" >/dev/null ; then
        python3 -c "import sys;sys.exit(1 if input('Not booted with toram - cannot write to CD.  Continue(y/n)?')!='y' else 0)" || exit 1
    fi
    #if this fails, it's probably been done already, so ignore it
    patch -p1 -N -d / < /usr/local/share/chroot_local_mirror.patch || true
else
    SOURCE_PATH=/home/rnpalmer/Debian/gpglive
    FROMSELF_BUILD=false
    MIRROR_ROOT=/media/rnpalmer/transfer/aptmirror
    USER1=rnpalmer
    if host www.debian.org; then
        echo "Do not run live-build with networking enabled, as it may download executables without verifying them (#718225)"
        exit 1
    fi
fi
#do we have the dependencies to rebuild everything?
if dpkg-query -W live-build && dpkg-query -W python3 && dpkg-query -W python3-scipy && dpkg-query -W alsa-utils && dpkg-query -W equivs ; then
    FULL_BUILD=true
else
    if [ -e $SOURCE_PATH/config/includes.chroot/usr/local/share/fakenettools_0.0fake_all.deb -a -e $SOURCE_PATH/config/includes.chroot/usr/local/share/keysound.wav ]; then
        if dpkg-query -W live-build >/dev/null && dpkg-query -W python3 >/dev/null; then
            FULL_BUILD=false
        else
            echo "Missing dependencies - please install live-build python3"
            exit 1
        fi
    else
        echo "Missing dependencies - please install live-build python3 python3-scipy alsa-utils equivs"
        exit 1
    fi
fi
IMAGEROOT_PATH=/var/cache/live-build/image1
mkdir -p $IMAGEROOT_PATH
#if image has been built before, clean up
cd $IMAGEROOT_PATH
if [ -e config ]; then
    rm -rf config
    rm -rf auto
fi
#copy to build area
cp -r $SOURCE_PATH -T $IMAGEROOT_PATH
#these need dependencies not in the image, so are only run if these are available
if $FULL_BUILD; then true; else
    $IMAGEROOT_PATH/config/misc_source/build_fakenettools.sh $SOURCE_PATH
    su $USER1 -c "$IMAGEROOT_PATH/config/misc_source/create_mask_sound.py $SOURCE_PATH"
    #copy result to build area
    cp -r $SOURCE_PATH -T $IMAGEROOT_PATH
fi
#put a copy of the config inside the image
cp -r $SOURCE_PATH -T config/includes.chroot/usr/local/share/image1
#check mirror integrity
#while the online mirror_check_update.py run will have done this, the point of being able to build from an existing liveCD is to not trust the online system
if [ -e $MIRROR_ROOT/debian/ ] ; then true ; else
    echo "Local mirror not found - to create it, run mirror_check_update.py with networking enabled"
    exit 1
fi
su $USER1 -c $IMAGEROOT_PATH/config/includes.chroot/usr/local/bin/mirror_check_update.py
#build
cd $IMAGEROOT_PATH
lb clean
lb config -m file://$MIRROR_ROOT/debian/ --parent-mirror-chroot-security file://$MIRROR_ROOT/debian-security/
lb build
echo "Warning: live-build does not report failure in its exit code - check whether the above actually finished (last 2 steps are lb chroot_devpts remove then a do-nothing lb source), see mirror_check_update.py if the problem is a missing package"
#xorriso writing doesn't work from the desktop - something else is using the drive (file manager?)
if $FROMSELF_BUILD; then
    if python3 -c "import sys;sys.exit(1 if input('Write to disk (y/n)?')!='y' else 0)" ; then
        umount /media/cdrom || true
        xorriso -as cdrecord -v dev=/dev/sr0 blank=as_needed $IMAGEROOT_PATH/live-image-amd64.hybrid.iso
    fi
    echo "To write (again) later: xorriso -as cdrecord -v dev=/dev/sr0 blank=as_needed $IMAGEROOT_PATH/live-image-amd64.hybrid.iso"
    echo "To save image/log: mkdir -p /mnt/transfer/imagetest && cp $IMAGEROOT_PATH/* /mnt/transfer/imagetest"
fi
echo "To enter the not-quite-final-image chroot for testing (gpg-agent will not work there, see enter_test_mode.sh): sudo chroot $IMAGEROOT_PATH/chroot"

