#!/bin/sh
#This enforces checking full fingerprint (not just keyid) + name and email address(es), send encrypted to owner (not keyserver)
#caff (signing-party) is also in the image, but isn't very suitable for offline use (#814531) - caff -R --key-file $KEYFILE $1 sort of works along with cat-as-fake-sendmail, but generates a PGP/MIME(?) message, which Thunderbird's 'Edit as new message' can't handle
set -e
KEYFILE=/mnt/transfer/sid_debian_keyring.gpg
if [ $# -ne 2 ] ; then
    if [ $# -ne 3 ] ; then
        echo "Usage: $0 <full fingerprint> <full uid> <optional keyring file, defaults to sid_debian_keyring.gpg>"
        echo "e.g. $0 1234567890abcdef1234567890abcdef12345678 'Test key 2 - do not trust <test2@rnpalmer-laptop>' ~/othergpgtest/key_to_sign.gpg"
        exit 1
    else
        KEYFILE=$3
    fi
fi
if echo $1 | grep -E -e "^[0-9a-fA-F]{40}$" ; then true; else
    echo "First argument is not 40 hex digits - use full fingerprints (not key IDs) without 0x or spaces, and check for typos"
    exit 1
fi
if gpg --keyring $KEYFILE --list-keys ="$2" | grep -iE -e $1 >/dev/null ; then true; else
    echo "Key to be signed is not in $KEYFILE - check for typos, to use a different keyring pass filename as 3rd argument"
    exit 1
fi
if [ -e ~/tmpgpg ] ; then
    echo "~/tmpgpg already exists - if it is left over from this script crashing, shred+delete it"
    exit 1
fi
cp -r ~/.gnupg ~/tmpgpg
export GNUPGHOME=~/tmpgpg
gpg --keyring $KEYFILE --export $1 | gpg --import-options import-minimal --import
gpg --default-cert-level 2 --quick-sign-key $1 ="$2"
gpg --export $1 | gpg --armor --encrypt -r $1 > ~/tmpgpg/main.txt
echo "To: $2\nSubject: Signature for key $1\n\nAs recommended by https://wiki.debian.org/Keysigning, I am sending this to you (and not to a keyserver) to verify that you have the private key and access to the UID's email address.  If you have multiple UIDs, check those email addresses for the other signatures.\n\nTo import and publish:\n   gpg --decrypt | gpg --import\n   gpg --keyserver <url> --send-keys $1\n\n" > ~/tmpgpg/header.txt
#redirection filenames can't contain <>, even quoted
STRIPPEDNAME=$(echo $2 | tr -dc [:alnum:])
cat ~/tmpgpg/header.txt ~/tmpgpg/main.txt > "/mnt/transfer/to_sign/signature$1_${STRIPPEDNAME}.eml"
#clean up securely
shred -u ~/tmpgpg/*/*
rmdir ~/tmpgpg/*/
#these are sockets not regular files, and can't be shredded
rm -r ~/tmpgpg/S.gpg-agent*
shred -u ~/tmpgpg/*
rmdir ~/tmpgpg
