#!/usr/bin/python3
#If a binary .changes is needed (i.e. for NEW), delete 'source' from the changes_files glob
#We use bundles and not just copies because FAT filesystems don't support symlinks, which at least beignet contains
#Not using debcommit -r --sign-tags as it makes a new commit
#On the online side:
#before - git bundle create /media/rnpalmer/transfer/to_sign/beignet.bundle HEAD
#after - git pull --tags /media/rnpalmer/transfer/to_sign/beignet_tag.bundle tag debian/1.3.2-1
import glob
import subprocess
import re
import sys
import tempfile
import os.path
import os
tosign_dir=os.path.abspath('.')
if len(sys.argv)>2:
    raise ValueError("too many arguments")
if len(sys.argv)==2:
    sourcepkg_name=sys.argv[1]
else:
    sourcepkg_name='*'
changes_files=glob.glob(tosign_dir+'/'+sourcepkg_name+'_*source.changes')
bundle_files=glob.glob(tosign_dir+'/'+sourcepkg_name+'.bundle')
if len(changes_files)>1 or len(bundle_files)>1:
    raise ValueError("Multiple files found: "+str(changes_files)+str(bundle_files)+("please specify source package name" if '*' in sourcepkg_name else "please delete old versions"))
if len(changes_files)==0 and len(bundle_files)==0:
    raise ValueError("No files to sign found in "+tosign_dir+" (only source .changes accepted)")
changes_file=changes_files[0] if len(changes_files)>0 else None
bundle_file=bundle_files[0] if len(bundle_files)>0 else None
if not (changes_file and bundle_file):
    if input("Only "+("changes" if changes_file else "bundle (only source .changes accepted)")+" file found - continue?(y/n)")!='y':
        sys.exit(1)
if '*' in sourcepkg_name:
    if changes_file:
        sourcepkg_name=changes_file.split('/')[-1].split('_')[0]
        if bundle_file and bundle_file.split('/')[-1].split('.')[0]!=sourcepkg_name:
            raise ValueError("changes and bundle names do not match - please specify source package name")
    else:
        sourcepkg_name=bundle_file.split('/')[-1].split('.')[0]
with tempfile.TemporaryDirectory() as git_tmpdir:
    if bundle_file:
        subprocess.run(["git","clone",bundle_file],cwd=git_tmpdir,check=True)
        with open(git_tmpdir+'/'+sourcepkg_name+'/debian/changelog','r') as git_changelog:
            git_line1=git_changelog.readline()
        git_line1m=re.match('(.*?) \((.*?)\) (.*?); urgency=',git_line1)
        if not git_line1m or git_line1m.group(1)!=sourcepkg_name:
            raise ValueError("git changelog invalid or does not match package name: "+git_line1)
        if git_line1m.group(3)=='UNRELEASED':
            raise ValueError("git changelog unreleased: "+git_line1)
        git_version=git_line1m.group(2)
    if changes_file:
        with open(changes_file,'r') as changes_fd:
            for line in changes_fd:
                changes_m=re.fullmatch('(.*?): (.*?)\n',line)
                if changes_m and changes_m.group(1)=='Source' and changes_m.group(2)!=sourcepkg_name:
                    raise ValueError("changes package name does not match: "+line)
                if changes_m and changes_m.group(1)=='Distribution' and changes_m.group(2)=='UNRELEASED':
                    raise ValueError("changes is unreleased: "+line)
                if changes_m and changes_m.group(1)=='Version':
                    changes_version=changes_m.group(2)
        if bundle_file and git_version!=changes_version:
            raise ValueError("changes and git are different versions: "+git_version+'!='+changes_version)
        subprocess.run(["debsign","-k",os.getenv("DEBFULLNAME"),changes_file],check=True)
    else:
        #git -s can't unlock a key with pinentry-curses, so sign something else first
        subprocess.run(["gpg","--output",git_tmpdir+"/dummy.sig","--detach-sign","/usr/local/share/dummy_message.txt"],check=True)
    if bundle_file:
        tagname="debian/"+git_version.replace('~','_').replace(':','%')
        tagmsg=sourcepkg_name+" Debian release "+git_version
        subprocess.run(["git","-c","user.email="+os.getenv("DEBEMAIL"),"-c","user.name="+os.getenv("DEBFULLNAME"),"tag","-s","-m",tagmsg,tagname],cwd=git_tmpdir+'/'+sourcepkg_name,check=True)#GIT_AUTHOR_* don't seem to work
        subprocess.run(["git","bundle","create",bundle_file.replace('.bundle','_tag.bundle'),'HEAD..'+tagname],cwd=git_tmpdir+'/'+sourcepkg_name,check=True)
