#!/usr/bin/python3
import os
import subprocess
import sys
import glob
subprocess.run(["sync"],check=True)
key_locations=[f for f in ['/mnt/keys1/keys/','/mnt/keys2/keys/','/mnt/keys3/keys/','/media/cdrom/keys/'] if os.path.exists(f)]
key_fsroots=[f.rsplit('/keys/',1)[0] for f in key_locations]
running_key_location='/home/user/.gnupg'
public_key_file='/mnt/transfer/my_public_key.gpg'
if len(key_locations)>0:
    keys_consistent=True
    for f in key_locations:
        key_consistent1=subprocess.run(["diff","-qr",f+'/1',f+'/2']).returncode==0
        key_consistent2=subprocess.run(["diff","-qr",f+'/2/private-keys-v1.d',f+'/bak/private-keys-v1.d']).returncode==0
        if not key_consistent1:
            print(f+" /1 and /2 don't match")
            keys_consistent=False
        if not key_consistent2:
            print(f+" /2 and /bak private keys don't match")
    if not keys_consistent:
        raise OSError("One or more key media internally inconsistent")
    if os.path.exists(running_key_location):
        socket_files=glob.glob(running_key_location+'/S.gpg-agent*')
        if len(socket_files)>0:
            subprocess.run(["rm"]+socket_files,check=True)#these special files break the later steps
        keys_uptodate=[subprocess.run(["diff","-qr",f+'/1',running_key_location]).returncode==0 for f in key_locations]
        if not all(keys_uptodate):
            a=input("Keys have changed - copy to key media "+str(key_locations)+" ?(y/n)")
            if a=='y':
                for f in key_locations:
                    if 'cdrom' in f:
                        subprocess.run(["sudo","umount","/media/cdrom"])
                        subprocess.run(["xorriso","-abort_on","FATAL","-assert_volid","KEYS","FATAL","-dev","/dev/sr0","-volid","KEYS","-update_r","/home/user/.gnupg","/keys/1","-update_r","/home/user/.gnupg","/keys/2"],check=True)
                        subprocess.run(["sync"],check=True)
                        subprocess.run(["sudo","mount","/dev/sr0","-o","noexec,ro","/media/cdrom"],check=True)#needed for verification
                        for f2 in [f+'/1',f+'/2']:
                            if subprocess.run(["diff","-qr",f2,running_key_location]).returncode!=0:
                                raise OSError("Copied files "+str(f2)+" do not match original")
                    else:
                        for f2 in [f+'/1',f+'/2']:
                            subprocess.run(["cp","-r",running_key_location,"-T",f2],check=True)
                            subprocess.run(["sync"],check=True)
                            if subprocess.run(["diff","-qr",f2,running_key_location]).returncode!=0:
                                raise OSError("Copied files "+str(f2)+" do not match original")
                if os.path.exists(os.path.dirname(public_key_file)):
                    subprocess.run(["rm","-f",public_key_file],check=True)#avoid an 'overwrite?' prompt
                    subprocess.run(["gpg","--output",public_key_file,"--export"],check=True)
            else:
                subprocess.run(["sudo","umount"]+key_fsroots,check=True)
                sys.exit(1)
    else:
        print("Working copy of key ("+running_key_location+") not found")
#overwrite+remove the ramdisk copy
if len(sys.argv)<=1 or sys.argv[1]!='-y':
    a=input("Unmount media and wipe in-memory keys?(y/n)")
    if a=='n':
        sys.exit(1)
files_to_shred=[f for f in glob.glob(running_key_location+"/**/*",recursive=True) if os.path.isfile(f)]
if len(files_to_shred)>0:
    print("Shredding ",files_to_shred)
    subprocess.run(["shred"]+files_to_shred,check=True)
subprocess.run(["rm","-rf",running_key_location])
if len(key_locations)>0:
    subprocess.run(["sudo","umount"]+key_fsroots,check=True)
if '/media/cdrom0/keys' in key_locations:
    subprocess.run(["sudo","eject",'/dev/sr0'],check=True)

